VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Settings"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'Class name : clsComSettings
'Description : Provides similar functionality as the savesetting/getsetting functions
'              in VB, except for the fact that this class works under HKEY_LocalMachine, instead of the
'              HKEY_CurrentUser,wich made the savesetting totally unusable for COM servers used in web applications
'Author : Stefan De Prins
'Mail : stefan.de.prins@pandora.be

Option Explicit

Private Const REG_SZ As Long = 1
Private Const REG_DWORD As Long = 4

Private Const HKEY_CLASSES_ROOT = &H80000000
Private Const HKEY_CURRENT_USER = &H80000001
Private Const HKEY_LOCAL_MACHINE = &H80000002
Private Const HKEY_USERS = &H80000003

Private Const ERROR_NONE = 0
Private Const ERROR_BADDB = 1
Private Const ERROR_BADKEY = 2
Private Const ERROR_CANTOPEN = 3
Private Const ERROR_CANTREAD = 4
Private Const ERROR_CANTWRITE = 5
Private Const ERROR_OUTOFMEMORY = 6
Private Const ERROR_ARENA_TRASHED = 7
Private Const ERROR_ACCESS_DENIED = 8
Private Const ERROR_INVALID_PARAMETERS = 87
Private Const ERROR_NO_MORE_ITEMS = 259

Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_SET_VALUE = &H2
Private Const KEY_ALL_ACCESS = &H3F

Private Const REG_OPTION_NON_VOLATILE = 0

Private Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hKey As Long) As Long
Private Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal Reserved As Long, ByVal lpClass As String, ByVal dwOptions As Long, ByVal samDesired As Long, ByVal lpSecurityAttributes As Long, phkResult As Long, lpdwDisposition As Long) As Long
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, phkResult As Long) As Long
Private Declare Function RegQueryValueExString Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, ByVal lpData As String, lpcbData As Long) As Long
Private Declare Function RegQueryValueExLong Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, lpData As Long, lpcbData As Long) As Long
Private Declare Function RegQueryValueExNULL Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, ByVal lpData As Long, lpcbData As Long) As Long
Private Declare Function RegSetValueExString Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, ByVal lpValue As String, ByVal cbData As Long) As Long
Private Declare Function RegSetValueExLong Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, lpValue As Long, ByVal cbData As Long) As Long
Private Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" (ByVal hKey As Long, ByVal lpValueName As String) As Long
Private Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" (ByVal hKey As Long, ByVal lpSubKey As String) As Long

'////////////////////////////////////
'P U B L I C
'////////////////////////////////////

Public Function GetSetting(ByVal vsAppname As String, ByVal vsSection As String, ByVal vsEntry As String, Optional ByVal vsDefault As String = "") As String
Dim lsValue As String

    lsValue = QueryValue(HKEY_LOCAL_MACHINE, "SOFTWARE\" & vsAppname & "\" & vsSection, vsEntry)
    If lsValue = "" Then
        GetSetting = vsDefault
    Else
        GetSetting = lsValue
    End If
    
End Function

Public Sub SaveSetting(ByVal vsAppname As String, ByVal vsSection As String, ByVal vsEntry As String, ByVal vsValue As String)
Dim hKey As Long
Dim lPos As Long

    If RegOpenKeyEx(HKEY_LOCAL_MACHINE, _
        "SOFTWARE\" & vsAppname & "\" & vsSection, _
        0&, KEY_ALL_ACCESS, hKey) <> ERROR_NONE _
    Then
        Call CreateNewKey("SOFTWARE\" & vsAppname & "\" & vsSection, HKEY_LOCAL_MACHINE)
    End If
    Call RegCloseKey(hKey)

    Call SetKeyValue(HKEY_LOCAL_MACHINE, "SOFTWARE\" & vsAppname & "\" & vsSection, vsEntry, vsValue, REG_SZ)
    
End Sub

Public Function DeleteSetting(ByVal vsAppname As String, ByVal vsSection As String, ByVal vsEntry As String) As Boolean
Dim hKey As Long

    ' Open the key.
    If RegOpenKeyEx(HKEY_LOCAL_MACHINE, _
        "SOFTWARE\" & vsAppname & "\" & vsSection, _
        0&, KEY_ALL_ACCESS, hKey) <> ERROR_NONE _
    Then
        'error / key doesn't exist
        DeleteSetting = False
        Exit Function
    Else
        If RegDeleteValue(hKey, vsEntry) = ERROR_NONE Then
            'OK
            DeleteSetting = True
        Else
            'error /value doesn't exist
            DeleteSetting = False
        End If
    End If
    
    Call RegCloseKey(hKey)
    
End Function

Public Function DeleteAppEntries(ByVal vsAppname As String) As Boolean
    
    If RegDeleteKey(HKEY_LOCAL_MACHINE, "SOFTWARE\" & vsAppname) = ERROR_NONE Then
        DeleteAppEntries = True
    Else
        DeleteAppEntries = False
    End If
    
End Function

'////////////////////////////////////
'P R I V A T E
'////////////////////////////////////

Private Function SetValueEx(ByVal hKey As Long, sValueName As String, _
   lType As Long, vValue As Variant) As Long
       Dim lValue As Long
       Dim sValue As String
       Select Case lType
           Case REG_SZ
               sValue = vValue & Chr$(0)
               SetValueEx = RegSetValueExString(hKey, sValueName, 0&, _
                                              lType, sValue, Len(sValue))
           Case REG_DWORD
               lValue = vValue
               SetValueEx = RegSetValueExLong(hKey, sValueName, 0&, _
   lType, lValue, 4)
           End Select
End Function

Private Function QueryValueEx(ByVal lhKey As Long, ByVal szValueName As String, vValue As Variant) As Long
       Dim cch As Long
       Dim lrc As Long
       Dim lType As Long
       Dim lValue As Long
       Dim sValue As String

       On Error GoTo QueryValueExError

       lrc = RegQueryValueExNULL(lhKey, szValueName, 0&, lType, 0&, cch)
       If lrc <> ERROR_NONE Then GoTo QueryValueExExit

       Select Case lType
           Case REG_SZ:
               sValue = String(cch, 0)

   lrc = RegQueryValueExString(lhKey, szValueName, 0&, lType, _
   sValue, cch)
               If lrc = ERROR_NONE Then
                   vValue = Left$(sValue, cch - 1)
               Else
                   vValue = Empty
               End If
           Case REG_DWORD:
   lrc = RegQueryValueExLong(lhKey, szValueName, 0&, lType, _
   lValue, cch)
               If lrc = ERROR_NONE Then vValue = lValue
           Case Else
               'all other data types not supported
               lrc = -1
       End Select

QueryValueExExit:
       QueryValueEx = lrc
       Exit Function

QueryValueExError:
       Resume QueryValueExExit
End Function

Private Sub CreateNewKey(sNewKeyName As String, lPredefinedKey As Long)
       Dim hNewKey As Long
       Dim lRetVal As Long

       lRetVal = RegCreateKeyEx(lPredefinedKey, sNewKeyName, 0&, _
                 vbNullString, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, _
                 0&, hNewKey, lRetVal)
       RegCloseKey (hNewKey)
   End Sub

Private Sub SetKeyValue(ByVal DefKey As Long, sKeyName As String, sValueName As String, vValueSetting As Variant, lValueType As Long)
       Dim lRetVal As Long
       Dim hKey As Long

       
       lRetVal = RegOpenKeyEx(DefKey, sKeyName, 0, _
                                 KEY_SET_VALUE, hKey)
       lRetVal = SetValueEx(hKey, sValueName, lValueType, vValueSetting)
       RegCloseKey (hKey)
       
End Sub

Private Function QueryValue(ByVal DefKey As Long, sKeyName As String, sValueName As String) As String
       Dim lRetVal As Long
       Dim hKey As Long
       Dim vValue As Variant

       lRetVal = RegOpenKeyEx(DefKey, sKeyName, 0, KEY_QUERY_VALUE, hKey)
       lRetVal = QueryValueEx(hKey, sValueName, vValue)
       RegCloseKey (hKey)
       QueryValue = vValue
       
End Function
